/****************************************************************************
** $Id: rs_spline.cpp,v 1.3 2004/03/18 21:28:53 js Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include "rs_spline.h"

#include "rs_debug.h"
#include "rs_graphicview.h"
#include "rs_painter.h"
#include "rs_graphic.h"


/**
 * Constructor.
 */
RS_Spline::RS_Spline(RS_EntityContainer* parent,
                     const RS_SplineData& d)
        :RS_AtomicEntity(parent), data(d) {
    calculateBorders();
}



/**
 * Destructor.
 */
RS_Spline::~RS_Spline() {}




RS_Entity* RS_Spline::clone() {
    RS_Spline* l = new RS_Spline(*this);
    l->initId();
    return l;
}



void RS_Spline::calculateBorders() {
    minV = RS_Vector::minimum(data.startpoint, data.endpoint);
    maxV = RS_Vector::maximum(data.startpoint, data.endpoint);
}



RS_VectorSolutions RS_Spline::getRefPoints() {
    RS_VectorSolutions ret(data.startpoint, data.endpoint);
    return ret;
}


RS_Vector RS_Spline::getNearestEndpoint(const RS_Vector& coord,
                                        double* dist) {
    double dist1, dist2;
    RS_Vector* nearerPoint;

    dist1 = data.startpoint.distanceTo(coord);
    dist2 = data.endpoint.distanceTo(coord);

    if (dist2<dist1) {
        if (dist!=NULL) {
            *dist = dist2;
        }
        nearerPoint = &data.endpoint;
    } else {
        if (dist!=NULL) {
            *dist = dist1;
        }
        nearerPoint = &data.startpoint;
    }

    return *nearerPoint;
}



RS_Vector RS_Spline::getNearestPointOnEntity(const RS_Vector& coord,
        bool onEntity, double* dist, RS_Entity** entity) {

    if (entity!=NULL) {
        *entity = this;
    }

    if (dist!=NULL) {
        *dist = RS_MAXDOUBLE;
    }

    return RS_Vector(false);
}



RS_Vector RS_Spline::getNearestCenter(const RS_Vector& coord,
                                      double* dist) {

    if (dist!=NULL) {
        *dist = RS_MAXDOUBLE;
    }

    return RS_Vector(false);
}



RS_Vector RS_Spline::getNearestMiddle(const RS_Vector& coord,
                                      double* dist) {
    if (dist!=NULL) {
        *dist = RS_MAXDOUBLE;
    }

    return RS_Vector(false);
}



RS_Vector RS_Spline::getNearestDist(double distance,
                                    const RS_Vector& coord,
                                    double* dist) {
    if (dist!=NULL) {
        *dist = RS_MAXDOUBLE;
    }

    return RS_Vector(false);
}



/*RS_Vector RS_Spline::getNearestRef(const RS_Vector& coord,
                                 double* dist) {
    double d1, d2, d;
    RS_Vector p;
    RS_Vector p1 = getNearestEndpoint(coord, &d1);
    RS_Vector p2 = getNearestMiddle(coord, &d2);
 
    if (d1<d2) {
        d = d1;
        p = p1;
    } else {
        d = d2;
        p = p2;
    }
 
    if (dist!=NULL) {
        *dist = d;
    }
 
    return p;
}*/


double RS_Spline::getDistanceToPoint(const RS_Vector& coord,
                                     RS_Entity** entity,
                                     RS2::ResolveLevel /*level*/,
                                     double /*solidDist*/) {

    RS_DEBUG->print("RS_Spline::getDistanceToPoint");

    if (entity!=NULL) {
        *entity = this;
    }

    return RS_MAXDOUBLE;
}



void RS_Spline::moveStartpoint(const RS_Vector& pos) {
    data.startpoint = pos;
    calculateBorders();
}



void RS_Spline::moveEndpoint(const RS_Vector& pos) {
    data.endpoint = pos;
    calculateBorders();
}


void RS_Spline::move(RS_Vector offset) {
    data.startpoint.move(offset);
    data.endpoint.move(offset);
    calculateBorders();
}



void RS_Spline::rotate(RS_Vector center, double angle) {
    data.startpoint.rotate(center, angle);
    data.endpoint.rotate(center, angle);
    calculateBorders();
}



void RS_Spline::scale(RS_Vector center, RS_Vector factor) {
    data.startpoint.scale(center, factor);
    data.endpoint.scale(center, factor);
    calculateBorders();
}



void RS_Spline::mirror(RS_Vector axisPoint1, RS_Vector axisPoint2) {
    data.startpoint.mirror(axisPoint1, axisPoint2);
    data.endpoint.mirror(axisPoint1, axisPoint2);
    calculateBorders();
}



void RS_Spline::moveRef(const RS_Vector& ref, const RS_Vector& offset) {
    if (ref.distanceTo(data.startpoint)<1.0e-4) {
        moveStartpoint(data.startpoint+offset);
    }
    if (ref.distanceTo(data.endpoint)<1.0e-4) {
        moveEndpoint(data.endpoint+offset);
    }
}



/**
 * Todo: draw the spline, user patterns.
 */
void RS_Spline::draw(RS_Painter* painter, RS_GraphicView* view) {
    if (painter==NULL || view==NULL) {
        return;
    }

	RS_PtrList<RS_Vector> points = getPoints();

	RS_Vector* p;
	RS_Vector last;
	for (p = points.first(); p!=NULL; p=points.next()) {
	    painter->drawLine(view->toGui(last),
    	                  view->toGui(*p));
		last = *p;
	}
}


/**
 * @return The reference points of the spline.
 */
RS_PtrList<RS_Vector> RS_Spline::getPoints() {
	RS_PtrList<RS_Vector> points;
	points.setAutoDelete(true);
	/*
	points.append(new RS_Vector(10.0,10.0));
	points.append(new RS_Vector(15.0,20.0));
	points.append(new RS_Vector(20.0,0.0));
	*/
	return points;
}


/**
 * Dumps the spline's data to stdout.
 */
std::ostream& operator << (std::ostream& os, const RS_Spline& l) {
    os << " Spline: " << l.getData() << "\n";
    return os;
}


// EOF
